/* EntryFileDialog.java
 *
 * created: Mon Dec  7 1998
 *
 * This file is part of Artemis
 *
 * Copyright (C) 1998,1999,2000  Genome Research Limited
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Header: /nfs/disk222/yeastpub/Repository/powmap/diana/components/EntryFileDialog.java,v 1.22 2000/04/14 12:25:25 kmr Exp $
 */

package diana.components;

import uk.ac.sanger.pathogens.*;
import uk.ac.sanger.pathogens.embl.DocumentEntryFactory;
import uk.ac.sanger.pathogens.embl.ReadFormatException;
import uk.ac.sanger.pathogens.embl.EntryInformation;
import uk.ac.sanger.pathogens.embl.EntryInformationException;
import uk.ac.sanger.pathogens.embl.InvalidRelationException;
import uk.ac.sanger.pathogens.embl.InvalidKeyException;
import uk.ac.sanger.pathogens.embl.InvalidQualifierException;
import uk.ac.sanger.pathogens.embl.Key;
import uk.ac.sanger.pathogens.embl.Qualifier;
import uk.ac.sanger.pathogens.embl.QualifierInfo;
import uk.ac.sanger.pathogens.embl.QualifierInfoException;

import java.awt.*;
import java.io.*;

/**
 *  This class is a FileDialog that can read EMBL Entry objects.
 *
 *  @author Kim Rutherford
 *  @version $Id: EntryFileDialog.java,v 1.22 2000/04/14 12:25:25 kmr Exp $
 **/

public class EntryFileDialog extends FileDialog {
  /**
   *  Create a new EntryFileDialog component.
   *  @param owner The component where this dialog was created.
   *  @param entry_group The EntryGroup object where the newly read Entry
   *    should be put.
   **/
  public EntryFileDialog (Frame owner) {
    super (owner, "Select an EMBL file...", FileDialog.LOAD);
    this.owner = owner;
  }

  /**
   *  Return an embl.Entry object representing the file the user has selected
   *  with the dialog or null if the read failed for any reason.
   *  @param entry_information The EntryInformation to use when reading.  This
   *    supplies the list of valid keys and qualifiers.  If a key or qualifier
   *    is read that is incompatible with this EntryInformation object the
   *    EntryInformation will be changed to cope.
   *  @exception EntryInformationException Thrown if an Entry using the given
   *    EntryInformation object cannot contain the Key, Qualifier or
   *    Key/Qualifier combination of one of the features in the Document.
   **/
  public uk.ac.sanger.pathogens.embl.Entry
    getEntry (final EntryInformation entry_information) {

    show ();

    if (getDirectory () == null || getFile () == null) {
      return null;
    }

    final File file = new File (getDirectory (), getFile ());

    StringVector errors = new StringVector ();

    try {
      final FileDocument file_document = new FileDocument (file);

      while (errors.size () < 10) {
        try {
          if (file != null) {
            final uk.ac.sanger.pathogens.embl.Entry new_entry =
              DocumentEntryFactory.makeDocumentEntry (entry_information,
                                                      file_document);

            return new_entry;
          } else {
            new MessageDialog (owner, "creating new entry: " + getFile ());

            final uk.ac.sanger.pathogens.embl.Entry new_entry =
              DocumentEntryFactory.makeDocumentEntry (file_document);

            return new_entry;
          }
        } catch (EntryInformationException e) {
          final String message = e.getMessage ();

          errors.add (message);

          if (errors.size () == 1) {
            final YesNoDialog yes_no_dialog =
              new YesNoDialog (owner,
                               message + " - ignore error and continue?");

            if (!yes_no_dialog.getResult ()) {
              return null;
            }
          }

          handleOpenException (entry_information, e);

          // go around the loop again
        }
      }

      new MessageDialog (owner, "aborted - too many errors while reading: " +
                         getFile ());

    } catch (ReadFormatException e) {
      final String message =
        "failed to open " + file + ": " + e.getMessage () +
        " at line: " + e.getLineNumber ();
      System.out.println (message);
      new MessageDialog (owner, message);
    } catch (IOException e) {
      final String message = "failed to open " + file + ": " + e.getMessage ();
      System.out.println (message);
      new MessageDialog (owner, message);
    } finally {
      if (errors.size () > 1) {
        final YesNoDialog dialog =
          new YesNoDialog (owner, "There were warnings while reading.  " +
                           "View them now?");

        if (dialog.getResult ()) {
          final FileViewer viewer =
            new FileViewer ("Read errors from " + file.getName ());

          final StringBuffer buffer = new StringBuffer ();

          for (int i = 0 ; i < errors.size () ; ++i) {
            buffer.append (errors.elementAt (i) + "\n");
          }

          viewer.readString (buffer.toString ());
        }
      }
    }

    return null;
  }

  /**
   *  Fix entry_information so that the given exception won't happen again.
   **/
  public static void
    handleOpenException (final EntryInformation entry_information,
                         final EntryInformationException exception) {
    if (exception instanceof InvalidKeyException) {
      final InvalidKeyException key_exception =
        (InvalidKeyException) exception;

      entry_information.addKey (key_exception.getKey ());
    }

    if (exception instanceof InvalidRelationException) {
      final InvalidRelationException relation_exception =
        (InvalidRelationException) exception;

      if (relation_exception.getQualifier () == null) {
        final Key key_to_add = relation_exception.getKey ();

        entry_information.addKey (key_to_add);
      } else {
        final Qualifier qualifier = relation_exception.getQualifier ();

        // change the QualifierInfo object in the EntryInformation so that it
        // can handle this key/qualifier combination

        final QualifierInfo qualifier_info =
          entry_information.getQualifierInfo (qualifier.getName ());

        final QualifierInfo new_qualifier_info;

        if (qualifier_info == null) {
          new_qualifier_info =
            new QualifierInfo (qualifier.getName (),
                               QualifierInfo.UNKNOWN, null, null, false);
        } else {
          new_qualifier_info =
            new QualifierInfo (qualifier_info.getName (),
                               qualifier_info.getType (), null, null, false);
        }

        try {
          entry_information.addQualifierInfo (new_qualifier_info);
        } catch (QualifierInfoException e) {
          throw new Error ("internal error - unexpected exception: " + e);
        }
      }
    }

    if (exception instanceof InvalidQualifierException) {
      final String exception_qualifier_name =
        ((InvalidQualifierException) exception).getQualifier ().getName ();

      // add it again, but make it less restrictive
      final QualifierInfo new_info =
        new QualifierInfo (exception_qualifier_name, QualifierInfo.UNKNOWN,
                           null, null, false);

      try {
        entry_information.addQualifierInfo (new_info);
      } catch (QualifierInfoException e) {
        throw new Error ("internal error - unexpected exception: " + e);
      }
    }
  }

  /**
   *  The Frame reference that was passed to the constructor.
   **/
  private Frame owner = null;
}


